/*
 * AgentDirectionFinder.cpp
 *
 *  Created on: Apr 19, 2013
 *      Author: ondro
 */



#include "AgentDirectionFinder.h"
using namespace cv;

void AgentDirectionFinder::init(string args){
	value = 100;
	debug_level = logDEBUG;
	// Later we want to use trigger, now for testing we are using timer
	//trigger_attach("AGENT_DIRECTION_FINDER_ACTIVITY", TRIGGER_MATCHING);
	//activity_dflt = ACTIVITY::PASSIVE;
	correct_vp_existed_dflt = false;
	command_left = STEER_LEFT_BACKWARD_RIGHT_FORWARD;
	command_right = STEER_LEFT_FORWARD_RIGHT_BACKWARD;
	command_forward = STEER_BOTH_FORWARD;
	command_stop = STEER_STOP;


	timer_attach(1000, 100);
	phase_dflt = PHASE::P0;
	//activity_dflt = ACTIVITY::ACTIVE;
	activity_dflt = ACTIVITY::PASSIVE;
	movement_dflt = MOVEMENT::NONE;


    time1 = 9000;
    time2 = 9000;
	time3 = 20000;
	time4 = 8000;
	termination_dflt = AGENTSPACE_RUNNING;
}

void AgentDirectionFinder::sense_select_act(int pid) {
	syn.lock();
	termination = space_read("termination", termination_dflt);
	activity = space_read("AGENT_DIRECTION_FINDER_ACTIVITY", activity_dflt);
	phase = space_read("AGENT_DIRECTION_FINDER_PHASE", phase_dflt);
	movement = space_read("AGENT_DIRECTION_FINDER_MOVEMENT", movement_dflt);
	correct_vp_existed = space_read("CORRECT_VP_EXISTED_bool",correct_vp_existed_dflt);
	syn.unlock();


	if (activity == ACTIVITY::ACTIVE)
		FILE_LOG(debug_level) << "DF active";
	else
		FILE_LOG(debug_level) << "DF passive";
	FILE_LOG(debug_level) << "Phase = " << phase;
	if (activity == ACTIVITY::ACTIVE){
		switch(phase){
		case PHASE::P0:{
			FILE_LOG(debug_level) << "Direction Finder phase 0";
			movement = MOVEMENT::RIGHT;
			phase = PHASE::P1;
			syn.lock();
			bool save_image = false;
			space_write("SAVE_IMAGE_bool", save_image);
			space_write("AGENT_DIRECTION_FINDER_PHASE", phase);
			space_write("CHASSIS_COMMAND_STRING", command_right, time1, 4);
			space_write("AGENT_DIRECTION_FINDER_MOVEMENT", movement, time1+400);
			syn.unlock();
		}
		break;
		case PHASE::P1:{
			FILE_LOG(debug_level) << "Direction Finder phase 1";
			if (movement == MOVEMENT::RIGHT){
				// robot is turning to the right
			}
			if (movement == MOVEMENT::NONE){
				if (!correct_vp_existed){
					movement = MOVEMENT::LEFT;
					phase = PHASE::P2;
					syn.lock();
					space_write("AGENT_DIRECTION_FINDER_PHASE", phase);
					space_write("CHASSIS_COMMAND_STRING", command_left, time2, 4);
					space_write("AGENT_DIRECTION_FINDER_MOVEMENT", movement, time2 + 200);
					syn.unlock();
				}
				else{
					FILE_LOG(debug_level) << "VP found on left side";
					phase = PHASE::P0;
					ACTIVITY::e agent_passive = ACTIVITY::PASSIVE;
					syn.lock();
					bool save_image = true;
					space_write("SAVE_IMAGE_bool", save_image);
					space_write("AGENT_DIRECTION_FINDER_PHASE", phase);
					space_write("AGENT_DIRECTION_FINDER_ACTIVITY", agent_passive, 4000, 2);
					syn.unlock();
				}
			}
		}
		break;
		case PHASE::P2:{
			FILE_LOG(debug_level) << "Direction Finder phase 2";
			if (movement == MOVEMENT::LEFT){
				// agent is turning to the left
			}
			if (movement == MOVEMENT::NONE){
				// robot is adjusting its position
				phase = PHASE::P3;
				ACTIVITY::e activity_passive = ACTIVITY::PASSIVE;
				ACTIVITY::e activity_active = ACTIVITY::ACTIVE;
				syn.lock();
				space_write("AGENT_MATCHER_ACTIVITY", activity_active);
				space_write("AGENT_DIRECTION_FINDER_PHASE", phase);
				// other agent (Agent_Matcher or Agent_Correction) must wake up this agent to activity
				space_write("AGENT_DIRECTION_FINDER_ACTIVITY", activity_passive);
				syn.unlock();
			}
		}
		break;
		case PHASE::P3:{
			if (movement == MOVEMENT::NONE){
				// position of robot is adjusted
				FILE_LOG(debug_level) << "Direction Finder phase 3";
				movement = MOVEMENT::LEFT;
				phase = PHASE::P4;
				syn.lock();
				bool save_image = false;
				space_write("SAVE_IMAGE_bool", save_image);
				space_write("AGENT_DIRECTION_FINDER_PHASE", phase);
				space_write("CHASSIS_COMMAND_STRING", command_left, time1, 4);
				space_write("AGENT_DIRECTION_FINDER_MOVEMENT", movement, time1+400);
				syn.unlock();
			}
		}
		break;
		case PHASE::P4:{
			FILE_LOG(debug_level) << "Direction Finder phase 4";
			if (movement == MOVEMENT::LEFT){
				// robot is turning to the left
			}
			if (movement == MOVEMENT::NONE){
				//
				if (!correct_vp_existed){
					movement = MOVEMENT::RIGHT;
					phase = PHASE::P5;
					syn.lock();
					space_write("AGENT_DIRECTION_FINDER_PHASE", phase);
					space_write("CHASSIS_COMMAND_STRING", command_right, time2+200, 4);
					space_write("AGENT_DIRECTION_FINDER_MOVEMENT", movement, time2 + 200);
					syn.unlock();
				}
				else{
					FILE_LOG(debug_level) << "VP found on right side";
					phase = PHASE::P0;
					ACTIVITY::e agent_passive = ACTIVITY::PASSIVE;
					syn.lock();
					bool save_image = true;
					space_write("SAVE_IMAGE_bool", save_image);
					space_write("AGENT_DIRECTION_FINDER_PHASE", phase);
					space_write("AGENT_DIRECTION_FINDER_ACTIVITY", agent_passive,4000,2);
					syn.unlock();
				}
			}
		}
		break;
		case PHASE::P5:{
			FILE_LOG(debug_level) << "Direction Finder phase 5";
			if (movement == MOVEMENT::RIGHT){
				// robot is turning to the right
			}
			if (movement == MOVEMENT::NONE){
				// robot is going to adjust its direction by matching
				phase = PHASE::P6;
				ACTIVITY::e activity_passive = ACTIVITY::PASSIVE;
				ACTIVITY::e activity_active = ACTIVITY::ACTIVE;
				syn.lock();
				space_write("AGENT_MATCHER_ACTIVITY", activity_active);
				space_write("AGENT_DIRECTION_FINDER_PHASE", phase);
				// other agent (Agent_Matcher or Agent_Correction) must wake up this agent to activity
				space_write("AGENT_DIRECTION_FINDER_ACTIVITY", activity_passive);
				syn.unlock();


				/*
				movement = MOVEMENT::FORWARD;
				phase = PHASE::P6;
				syn.lock();
				space_write("AGENT_DIRECTION_FINDER_PHASE", phase);
				space_write("CHASSIS_COMMAND_STRING", command_forward, time3, 4);
				space_write("AGENT_DIRECTION_FINDER_MOVEMENT", movement, time3);
				syn.unlock();
				 */
			}
		}
		break;
		case PHASE::P6:{
			if (movement == MOVEMENT::NONE){
				// direction of robot is adjusted and robot is going forward
				FILE_LOG(debug_level) << "Direction Finder phase 6";
				movement = MOVEMENT::FORWARD;
				phase = PHASE::P7;
				syn.lock();
				bool save_image = false;
				space_write("SAVE_IMAGE_bool", save_image);
				space_write("AGENT_DIRECTION_FINDER_PHASE", phase);
				space_write("CHASSIS_COMMAND_STRING", command_forward, time3, 4);
				space_write("AGENT_DIRECTION_FINDER_MOVEMENT", movement, time3);
				syn.unlock();
			}
		}
		break;
		case PHASE::P7:{
			FILE_LOG(debug_level) << "Direction Finder phase 7";
			if (movement == MOVEMENT::FORWARD){
				// robot is going forward
			}
			if (movement == MOVEMENT::NONE){
				phase = PHASE::P0;
				//ACTIVITY::e activity_temp = ACTIVITY::PASSIVE;
				syn.lock();
				//space_write("AGENT_DIRECTION_FINDER_ACTIVITY", activity_temp);
				bool get_ref_image_once = true;
				space_write("REFERENCE_IMAGE_ONCE_bool", get_ref_image_once);
				space_write("AGENT_DIRECTION_FINDER_PHASE", phase);
				//space_write("CHASSIS_COMMAND_STRING", command_right, time4, 4);
				//space_write("AGENT_DIRECTION_FINDER_MOVEMENT", movement, time4);
				syn.unlock();
			}
		}
		break;
		default:{
		}
		break;
		}
	}
	else {
		// do nothing
	}




	if (termination == AGENTSPACE_TERMINATED){
		kill();
	}
}

AgentDirectionFinder::AgentDirectionFinder(string args) : Agent(args) {
	name = "AgentDirectionFinder";
}

AgentDirectionFinder::AgentDirectionFinder(Config cfg) : Agent(cfg) {
	name = "AgentDirectionFinder";
}
