/*
 * AgentMatcher.cpp
 *
 *  Created on: Apr 26, 2013
 *      Author: ondro
 */

#include "AgentMatcher.h"
using namespace cv;

void AgentMatcher::init (string args) {
	debug_level = logDEBUG;
	image_reference_dflt = imread("lena.jpg");
	image_dflt = imread("lena.jpg");
	correct_vp_dflt = false;
	activity_dflt = ACTIVITY::PASSIVE;
    correction_time = 500;

	termination_dflt = AGENTSPACE_RUNNING;

	// movement
	command_left = STEER_LEFT_BACKWARD_RIGHT_FORWARD;
	command_right = STEER_LEFT_FORWARD_RIGHT_BACKWARD;

	timer_attach(1000, 200);
	trigger_attach("AGENT_MATCHER_ACTIVITY", TRIGGER_MATCHING);

	//trigger_attach("CORRECT_VP_bool", TRIGGER_MATCHING);
}
void AgentMatcher::sense_select_act (int pid) {
	syn.lock();
	termination = space_read("termination", termination_dflt);
	activity = space_read("AGENT_MATCHER_ACTIVITY", activity_dflt);
	image_reference = space_read("REFERENCE_IMAGE_Mat", image_reference_dflt);
	image = space_read("INPUT_Mat", image_dflt);
	image = image.clone();
	syn.unlock();

	if (activity == ACTIVITY::ACTIVE){
		FILE_LOG(logDEBUG) << "AgentMatcher";

		// homography matrix finds images of corners of the reference image in scanned image
		match(image_reference, image, img_matches, scene_corners);

		FILE_LOG(debug_level) << "image corners:";
		for(unsigned int i = 0; i < scene_corners.size(); i++){
			FILE_LOG(debug_level) << "    " << i << ". point " << scene_corners[i].x << " " << scene_corners[i].y;
		}
		// Area of the rectangle formed by images of corners;
		double image_area = contourArea(scene_corners);
		FILE_LOG(debug_level) << "Area = " << image_area;

		// Area of the reference image
		vector<Point2f> reference_corners;
		reference_corners.push_back(Point2f(0,0));
		reference_corners.push_back(Point2f(image_reference.cols, 0));
		reference_corners.push_back(Point2f(image_reference.cols, image_reference.rows));
		reference_corners.push_back(Point2f(0, image_reference.rows));
		double reference_area = contourArea(reference_corners);

		// Check whether the area of the transformed image is not degenerated
		bool degenerated_image = false;
		if (image_area < (0.5 * reference_area) || reference_area < (0.5 * image_area)){
			degenerated_image = true;
		}

		// Check whether command to correction of robot was sent
		bool command_sent = false;
		MOVEMENT::e agent_next_movement = MOVEMENT::NONE;
		if (!degenerated_image){
			// We need to turn to right side
			if (scene_corners[0].x < -20 && scene_corners[3].x < -20){
				syn.lock();
				command_sent = true;
				agent_next_movement = MOVEMENT::LEFT;

				syn.unlock();
			}
			// We need to turn to left side
			else if (scene_corners[0].x > 20 && scene_corners[3].x > 20){
				syn.lock();
				command_sent = true;
				agent_next_movement = MOVEMENT::RIGHT;

				syn.unlock();
			}
			else {
				FILE_LOG(debug_level) << "Agent Matcher No correction";
			}
		}
		else{
			FILE_LOG(debug_level) << "Agent Matcher: Degenerated image!, no signal sent to chassis";
		}

		img_write = img_matches.clone();
		syn.lock();
		space_write("MATCHED_Mat", img_write);
		if (command_sent){
			if (agent_next_movement == MOVEMENT::RIGHT){
				FILE_LOG(debug_level) << "Agent Matcher moving to the right side";
				space_write("CHASSIS_COMMAND_string", command_right, correction_time, 4);
			}
			else if (agent_next_movement == MOVEMENT::LEFT){
				FILE_LOG(debug_level) << "Agent Matcher moving to the left side";
				space_write("CHASSIS_COMMAND_string", command_left, correction_time, 4);
			}
			space_write("POSITION_CORRECTION_bool", command_sent, correction_time);
			ACTIVITY::e agent_passive = ACTIVITY::PASSIVE;
			space_write("AGENT_MATCHER_ACTIVITY", agent_passive);
			bool save_image = true;
			space_write("SAVE_IMAGE_bool", save_image);
		}
		// We want to wake up direction_finder to activity
		else{
			FILE_LOG(debug_level) << "Matcher waking up direction finder";
			ACTIVITY::e agent_active = ACTIVITY::ACTIVE;
			ACTIVITY::e agent_passive = ACTIVITY::PASSIVE;
			space_write("AGENT_DIRECTION_FINDER_ACTIVITY", agent_active, 0, 2);
			space_write("AGENT_MATCHER_ACTIVITY", agent_passive);
			bool save_image = true;
			space_write("SAVE_IMAGE_bool", save_image);
		}
		syn.unlock();
	}
	else{
		// do nothing
	}

	if (termination == AGENTSPACE_TERMINATED){
		kill();
	}
}

AgentMatcher::AgentMatcher (string args) : Agent(args) {
	name = "AgentMatcher";
};
AgentMatcher::AgentMatcher (Config& cfg) : Agent(cfg){
	name = "AgentMatcher";
}
void AgentMatcher::match(cv::Mat& img1, cv::Mat& img2, cv::Mat& img_matches,
		vector<cv::Point2f>& scene_corners) {
	Mat img_object = img1;
	Mat img_scene = img2;
	//-- Localize the object
	std::vector<Point2f> obj;
	std::vector<Point2f> scene;

	if( !img_object.data || !img_scene.data )
	{ FILE_LOG(debug_level)<< " --(!) Error reading images " << std::endl; }

	//-- Step 1: Detect the keypoints using SURF Detector
	int minHessian = 400;

	SiftFeatureDetector detector( minHessian );

	std::vector<KeyPoint> keypoints_object, keypoints_scene;

	detector.detect( img_object, keypoints_object );
	detector.detect( img_scene, keypoints_scene );
	//FILE_LOG(debug_level) << "Tu somA";

	//-- Step 2: Calculate descriptors (feature vectors)
	//SurfDescriptorExtractor extractor;
	SiftDescriptorExtractor extractor;

	Mat descriptors_object, descriptors_scene;

	extractor.compute( img_object, keypoints_object, descriptors_object );
	extractor.compute( img_scene, keypoints_scene, descriptors_scene );
	//FILE_LOG(debug_level) << "TU somB";

	//cout << keypoints_scene.size() << " " << keypoints_object.size() << endl;

	//-- Step 3: Matching descriptor vectors using FLANN matcher
	//FILE_LOG(debug_level) << descriptors_object.cols << " " << descriptors_object.rows << " " << descriptors_scene.cols << " " << descriptors_scene.rows;
	FlannBasedMatcher matcher;
	std::vector< DMatch > matches;
	if (descriptors_object.data && descriptors_scene.data){
		matcher.match( descriptors_object, descriptors_scene, matches );




		double max_dist = 0; double min_dist = 100;

		//-- Quick calculation of max and min distances between keypoints
		for( int i = 0; i < descriptors_object.rows; i++ )
		{ double dist = matches[i].distance;
		if( dist < min_dist ) min_dist = dist;
		if( dist > max_dist ) max_dist = dist;
		}

		//FILE_LOG(debug_level) << max_dist;
		//FILE_LOG(debug_level) << min_dist;
		//printf("-- Max dist : %f \n", max_dist );
		//printf("-- Min dist : %f \n", min_dist );

		//-- Draw only "good" matches (i.e. whose distance is less than 3*min_dist )
		std::vector< DMatch > good_matches;

		for( int i = 0; i < descriptors_object.rows; i++ )
		{ if( matches[i].distance <= 3*min_dist )
		{ good_matches.push_back( matches[i]); }
		}

		//Mat img_matches;
		drawMatches( img_object, keypoints_object, img_scene, keypoints_scene,
				good_matches, img_matches, Scalar::all(-1), Scalar::all(-1),
				vector<char>(), DrawMatchesFlags::NOT_DRAW_SINGLE_POINTS );

		//-- Localize the object

		for( unsigned int i = 0; i < good_matches.size(); i++ )
		{
			//-- Get the keypoints from the good matches
			obj.push_back( keypoints_object[ good_matches[i].queryIdx ].pt );
			scene.push_back( keypoints_scene[ good_matches[i].trainIdx ].pt );
		}


		FILE_LOG(debug_level) << " velkosti: "<< obj.size() << " " << scene.size();
	}

	if (obj.size()>=4 && scene.size()>=4){

		Mat H = findHomography( obj, scene, CV_RANSAC );

		//-- Get the corners from the image_1 ( the object to be "detected" )
		std::vector<Point2f> obj_corners(4);
		obj_corners[0] = cvPoint(0,0); obj_corners[1] = cvPoint( img_object.cols, 0 );
		obj_corners[2] = cvPoint( img_object.cols, img_object.rows ); obj_corners[3] = cvPoint( 0, img_object.rows );
		//std::vector<Point2f> scene_corners(4);

		perspectiveTransform( obj_corners, scene_corners, H);

		//-- Draw lines between the corners (the mapped object in the scene - image_2 )
		line( img_matches, scene_corners[0] + Point2f( img_object.cols, 0), scene_corners[1] + Point2f( img_object.cols, 0), Scalar(0, 255, 0), 4 );
		line( img_matches, scene_corners[1] + Point2f( img_object.cols, 0), scene_corners[2] + Point2f( img_object.cols, 0), Scalar( 0, 255, 0), 4 );
		line( img_matches, scene_corners[2] + Point2f( img_object.cols, 0), scene_corners[3] + Point2f( img_object.cols, 0), Scalar( 0, 255, 0), 4 );
		line( img_matches, scene_corners[3] + Point2f( img_object.cols, 0), scene_corners[0] + Point2f( img_object.cols, 0), Scalar( 0, 255, 0), 4 );

	}
}

AgentMatcher::~AgentMatcher(){};


