/*
 * AgentPNGRead.cpp
 *
 *  Created on: Apr 6, 2013
 *      Author: ondro
 */

#include "AgentPNGRead.h"
using namespace cv;
using namespace std;
using namespace boost::filesystem;

void AgentPNGRead::init(string args) {
	FILE_LOG(logDEBUG2) << "Init of AgentPNGRead";
	path pp(DEFAULT_INPUT_PNG_FILES_DIR);
	p = pp;
	//FILE_LOG(logDEBUG) << p.string();
	try
	{
		if (exists(p))    // does p actually exist?
		{
			if (is_regular_file(p))        // is p a regular file?
				cout << p << " size is " << file_size(p) << '\n';

			else if (is_directory(p))      // is p a directory?
			{
				//cout << p << " is a directory containing:\n";
				// store paths, so we can sort them later
				copy(directory_iterator(p), directory_iterator(), back_inserter(files));
				sort(files.begin(), files.end());             // sort, since directory iteration
				// is not ordered on some file systems
				/*
				for (vector_path::const_iterator it(v.begin()), it_end(v.end()); it != it_end; ++it)
				{
					cout << "   " << *it << '\n';
				}
				*/
			}
			else
				cout << p << " exists, but is neither a regular file nor a directory\n";
		}
		else
			cout << p << " does not exist\n";
	}

	catch (const filesystem_error& ex)
	{
		cout << ex.what() << '\n';
	}

	counter = 0;
	timer_attach(1000, 1000/10);
}

void AgentPNGRead::sense_select_act(int pid) {
	if (counter < files.size()){
		FILE_LOG(logDEBUG2) << "start of reading file..";
		image = imread(files[counter].string());
		image = image.clone();
		syn.lock();
		size = image.size();
		space_write("IMAGE_SIZE_Size", size, 5000);
		space_write("INPUT_Mat", image, 2000);
		syn.unlock();
		FILE_LOG(logDEBUG2) << image.rows << " " << image.cols;
		counter++;
	}
	syn.lock();
	// block "termination" is used for propagation signal for end of agentspace (because of user termination or critical error)
	termination = space_read("termination", termination_dflt);
	syn.unlock();
	if (termination == AGENTSPACE_TERMINATED) {
		kill();
	}
}

AgentPNGRead::AgentPNGRead(string args) : Agent(args) {
	cfg.width = DEFAULT_WIDTH;
	cfg.height = DEFAULT_HEIGHT;
	name = "AgentPNGRead";
}

AgentPNGRead::AgentPNGRead(Config& cfg) : Agent(cfg) {
	name = "AgentPNGRead";
}

AgentPNGRead::~AgentPNGRead() {
}


