/*
 * AgentVanishingPoint.cpp
 *
 *  Created on: Apr 6, 2013
 *      Author: ondro
 */


#include "AgentVanishingPoint.h"
using namespace cv;

void AgentVanishingPoint::init(string args){
	termination_dflt = AGENTSPACE_RUNNING;
	numVps = 1;
	vanishingPoint.init(numVps, cfg.width, cfg.height);
	image_dflt = imread("lena.jpg");

	middle_region_left_dflt = -1;
	middle_region_right_dflt = -1;
	middle_region_height_dflt = -1;
	minThreshold_dflt = 120;
	maxThreshold_dflt = 180;

	//timer_attach(1000,100);
	//inputBlockName = "INPUT_LAPLACIAN_Mat";
	inputBlockName = "INPUT_Mat";
	//trigger_attach(inputBlockName, TRIGGER_MATCHING);
	timer_attach(2000, 100);
}

void AgentVanishingPoint::sense_select_act(int pid){
	syn.lock();
	termination = space_read("termination", termination_dflt);
	image = space_read(inputBlockName, image_dflt);
	minThreshold = space_read("Trackbar_lowThreshold_int", minThreshold_dflt);
	maxThreshold = space_read("Trackbar_maxThreshold_int", maxThreshold_dflt);
	middle_region_left = space_read("MIDDLE_REGION_LEFT_float", middle_region_left_dflt);
	middle_region_right = space_read("MIDDLE_REGION_RIGHT_float", middle_region_right_dflt);
	middle_region_height = space_read("MIDDLE_REGION_HEIGHT_float", middle_region_height_dflt);
	//line_segments = space_read("lSegs", line_segments_dflt);
	inputImg = image.clone();
	syn.unlock();



	syn.lock();
	//image = space_read("Work2_Mat", image_dflt);
	vanishingPoint.setMinThreshold(minThreshold);
	vanishingPoint.setMaxThreshold(maxThreshold);
	FILE_LOG(logDEBUG1) << "minThreshold = " << minThreshold << " maxThreshold = " << maxThreshold;
	syn.unlock();

	// Color Conversion
	if (inputImg.channels() == 3){
		cvtColor(inputImg, imgGRAY, CV_BGR2GRAY);
		inputImg.copyTo(outputImg);
	}
	else{
		inputImg.copyTo(imgGRAY);
		cvtColor(inputImg, outputImg, CV_GRAY2BGR);
	}
	vector<cv::Mat> vps; // Vanishing points
	vanishingPoint.processImage(numVps, imgGRAY, outputImg, vps); // find vanishing points
	vanishingPoint.getlineSegmentsClusters(line_segments);

	//if (line_segments.size() != 0){
	//	FILE_LOG(logDEBUG) << "************ lineSegmentsCluster[0].size() = " << line_segments[0].size();
	//}





	/*
	FILE_LOG(logDEBUG) << "Line segment size = " << line_segments.size();
	for(unsigned int i = 0; i < line_segments.size(); i++){
		line(outputImg, line_segments[i][0], line_segments[i][1], CV_RGB(0,255,0), 1);
	}
	 */


	// take just first vanishing point detected
	Point2f point;
	// -1 stands for non-existing vanishing point
	point.x = -1;
	point.y = -1;
	bool correct_vp = true;
	if (vps.size() > 0){
		point.x = vps.front().at<float>(0,0);
		point.y = vps.front().at<float>(1,0);
		//point.z = vps.front().at<float>(2,0);
		FILE_LOG(logDEBUG4) << "First Vanishing Point is [" << point.x << ", " << point.y << "]";
	}
	// Correction of vanishing point; checking whether is in the image
	if (isnan(point.x) || isnan(point.y)){
		correct_vp = false;
	}
	else{
		if (point.x < 0 || point.x > inputImg.size().width || point.y < 0 || point.y > inputImg.size().height){
			correct_vp = false;
		}
	}
	if (!correct_vp){
		point.x = -1;
		point.y = -1;
	}

	//draw boundaries for vanishing point
	Point2f point_rectangle_1(middle_region_left, 0);
	Point2f point_rectangle_2(middle_region_right, middle_region_height);
	rectangle(outputImg, point_rectangle_1, point_rectangle_2, CV_RGB(0,255,0), 2);

	/*
	// draw star around vanishing point
	int best_angle = -1;
	int best_sum = INT_MAX;
	if (line_segments.size() > 0 && (point.x != -1) && (point.y != -1)){
		for (int i = 0; i < 90; i++){
			Point point2;
			Point point1 = point;
			point2.x = point.x + 1000 * cos(CV_PI * (i / 180.0));
			point2.y = point.y + 1000 * sin(CV_PI * (i / 180.0));
			double a = point2.y - point1.y;
			double b = point1.x - point2.x;
			double c = - (a * point1.x + b * point1.y);
			double sum = 0;
			for(size_t j = 0; j < line_segments[0].size(); j++){
				Point p1 = line_segments[0][j][0];
				Point p2 = line_segments[0][j][1];

				if ((p1.x > point1.x && p1.y > point1.y) && (p2.x > point1.x && p2.y > point1.y)){
					double d1 = abs(a*p1.x + b*p1.y + c)/sqrt(a*a + b*b);
					double d2 = abs(a*p2.x + b*p2.y + c)/sqrt(a*a + b*b);
					sum += d1 + d2;
				}
			}
			if (sum < best_sum){
				best_sum = sum;
				best_angle = i;
			}

		}
		Point point3;
		point3.x = point.x + 1000 * cos(CV_PI * (best_angle / 180.0));
		point3.y = point.y + 1000 * sin(CV_PI * (best_angle / 180.0));

		line(outputImg, point, point3, CV_RGB(0, 255, 0),2,8);
		FILE_LOG(logDEBUG) << "Best right angle= " << best_angle;
	}

	best_angle = -1;
	best_sum = INT_MAX;
	if (line_segments.size() > 0 && (point.x != -1) && (point.y != -1)){
			for (int i = 90; i < 180; i++){
				Point point2;
				Point point1 = point;
				point2.x = point.x + 1000 * cos(CV_PI * (i / 180.0));
				point2.y = point.y + 1000 * sin(CV_PI * (i / 180.0));
				double a = point2.y - point1.y;
				double b = point1.x - point2.x;
				double c = - (a * point1.x + b * point1.y);
				double sum = 0;
				for(size_t j = 0; j < line_segments[0].size(); j++){
					Point p1 = line_segments[0][j][0];
					Point p2 = line_segments[0][j][1];

					if ((p1.x <= point1.x && p1.y > point1.y) && (p2.x <= point1.x && p2.y > point1.y)){
						double d1 = abs(a*p1.x + b*p1.y + c)/sqrt(a*a + b*b);
						double d2 = abs(a*p2.x + b*p2.y + c)/sqrt(a*a + b*b);
						sum += d1 + d2;
					}
				}
				if (sum < best_sum){
					best_sum = sum;
					best_angle = i;
				}
			}
			Point point3;
			point3.x = point.x + 1000 * cos(CV_PI * (best_angle / 180.0));
			point3.y = point.y + 1000 * sin(CV_PI * (best_angle / 180.0));
			line(outputImg, point, point3, CV_RGB(0, 255, 0),2,8);
			FILE_LOG(logDEBUG) << "Best left angle= " << best_angle;
		}
	*/


	FILE_LOG(logDEBUG4) << "Vanishing Point coordinates: " << point.x << " " << point.y;
	syn.lock();
	if (correct_vp){
		space_write("VANISHING_POINT_Point2f", point, 500);
	}
	// We write into space just correct VP;
	space_write("Vanishing_Mat", outputImg, 2000);
	space_write("IMG_GRAY", imgGRAY, 2000);
	space_write("IMG_CANNY", vanishingPoint.getimgCanny(), 2000);
	syn.unlock();
	if (termination == AGENTSPACE_TERMINATED){
		kill();
	}
}

AgentVanishingPoint::AgentVanishingPoint(string args) : Agent(args){
	name = "AgentVanishingPoint";
}
AgentVanishingPoint::AgentVanishingPoint(Config cfg) : Agent(cfg){
	procWidth = cfg.width;
	procHeight = cfg.height;
	name = "AgentVanishingPoint";
}
AgentVanishingPoint::~AgentVanishingPoint(){}
