/*
 * AgentVanishingPointControl.cpp
 *
 *  Created on: Apr 17, 2013
 *      Author: ondro
 */

#include "AgentVanishingPointControl.h"
using namespace cv;

void AgentVanishingPointControl::init(string args){
	debug_level = logDEBUG1;
	//trigger_attach("VANISHING_POINT_Point2f", TRIGGER_MATCHING);
	//trigger_attach("TEMP_VALUE_INT", TRIGGER_MATCHING);
	timer_attach(1000, 100);
	termination_dflt = AGENTSPACE_RUNNING;
	vp_dflt = Point2f(-1, -1);
	size_dflt = Size(-1, -1);
	// if vanishing point point is in middle column specified by middle_region, robot is going forward
	// interval is computed according following formula: <image_width/2 - (middle_region * image_width), image_width/2 + (middle_region * image_width)>
	middle_region_width_coef = 1/20.0;
	middle_region_height_coef = 3/4.0;

}

void AgentVanishingPointControl::sense_select_act(int pid) {
	syn.lock();
	// signalization for termination of the agent
	termination = space_read("termination", termination_dflt);
	// vanishing point
	vp = space_read("VANISHING_POINT_Point2f", vp_dflt);
	// size of image in which lies vanishing point
	size = space_read("IMAGE_SIZE_Size", size_dflt);
	syn.unlock();


	//FILE_LOG(logDEBUG) << middle_region;
	FILE_LOG(debug_level) << "VanishingPointAgentControl " << vp.x << " " << vp.y;
	FILE_LOG(debug_level) << "VanishingPointAgentControl " << size.height << " " << size.width;

	float middle_region_left = size.width/2 - (middle_region_width_coef * size.width);
	float middle_region_right = size.width/2 + (middle_region_width_coef * size.width);
	float middle_region_height = middle_region_height_coef * size.height;
	FILE_LOG(debug_level) << "middle region = < " << middle_region_left << ", " << middle_region_right << " > " << "height = " << middle_region_height;

	syn.lock();
	space_write("MIDDLE_REGION_LEFT_float", middle_region_left, 1000);
	space_write("MIDDLE_REGION_RIGHT_float", middle_region_right, 1000);
	space_write("MIDDLE_REGION_HEIGHT_float", middle_region_height, 1000);
	syn.unlock();

	// check whether vanishing point lies in the middle region
	bool correct_vp = false;
	if (vp != vp_dflt && size != size_dflt){
		if (vp.y <= middle_region_height){
			if (vp.x < middle_region_left){
				FILE_LOG(debug_level) << "Steering Left";
				command = STEER_LEFT_BACKWARD_RIGHT_FORWARD;
				syn.lock();
				space_write("CHASSIS_COMMAND_STRING", command, 250, 2);
				syn.unlock();
			}
			else if( middle_region_left <= vp.x && vp.x <= middle_region_right){
				correct_vp = true;
				FILE_LOG(debug_level) << "Steering Forward";
				command = STEER_BOTH_FORWARD;
				syn.lock();
				space_write("CHASSIS_COMMAND_STRING", command, 1000, 2);
				syn.unlock();
			}
			else if( middle_region_right < vp.x ){
				FILE_LOG(debug_level) << "Steering Right";
				command = STEER_LEFT_FORWARD_RIGHT_BACKWARD;
				syn.lock();
				space_write("CHASSIS_COMMAND_STRING", command, 250, 2);
				syn.unlock();
			}
		}
		else{
			FILE_LOG(debug_level) << "Vanishing Point is in lower part of the image.";
		}
	}

	if (correct_vp){
	syn.lock();
	space_write("CORRECT_VP_bool", correct_vp, 200);
	space_write("WAS_CORRECT_VP_bool", correct_vp, 3000);
	syn.unlock();
	}
	if (termination == AGENTSPACE_TERMINATED){
		kill();
	}
}

AgentVanishingPointControl::AgentVanishingPointControl(string args) : Agent(args) {
	name = "AgentVanishingPointControl";
}

AgentVanishingPointControl::AgentVanishingPointControl(Config cfg) : Agent(cfg) {
	name = "AgentVanishingPointControl";
}



