/*
 * VanishingPoint.cpp
 *
 *  Created on: Apr 6, 2013
 *      Author: ondro
 */

#include "VanishingPoint.h"
using namespace cv;
using namespace std;

void VanishingPoint::setMinThreshold(int minThreshold){
	minCannyThreshold = minThreshold;
}
void VanishingPoint::setMaxThreshold(int maxThreshold){
	maxCannyThreshold = maxThreshold;
}
Mat& VanishingPoint::getimgCanny(){
	return imgCanny;
}

void VanishingPoint::processImage(MSAC& msac, int numVps, Mat& imgGRAY, Mat& outputImg, vector<Mat>& vps){

	int kernelSize = 5;
	GaussianBlur(imgGRAY, imgGRAY, Size(kernelSize, kernelSize), 0);

	Canny(imgGRAY, imgCanny, minCannyThreshold, maxCannyThreshold, 3);
	int houghThreshold;
	//Hough
	vector<vector<Point> > lineSegments;
	vector<Point> aux;
	vector<Vec4i> lines;
	houghThreshold = 70;
	if(imgGRAY.cols*imgGRAY.rows < 400*400)
		houghThreshold = 100;



	cv::HoughLinesP(imgCanny, lines, 1, CV_PI/180, houghThreshold, 50, 5);
	while(lines.size() > 200)
	{
		lines.clear();
		houghThreshold += 20;
		cv::HoughLinesP(imgCanny, lines, 1, CV_PI/180, houghThreshold, 50,5);
	}

	//Selection of leaning lines (non horizontal non vertical)
	vector<float> angle;

	float angle_diff = 10;
	vector<Vec2f> thresholds;
	Vec2f vertical[4];
	Vec2f horizontal[4];

	horizontal[0][0] = 0;
	horizontal[0][1] = angle_diff;
	horizontal[1][0] = 180 - angle_diff;
	horizontal[1][1] = 180;
	horizontal[2][0] = 180;
	horizontal[2][1] = 180 + angle_diff;
	horizontal[3][0] = 360 - angle_diff;
	horizontal[3][1] = 360;

	vertical[0][0] = 90 - angle_diff;
	vertical[0][1] = 90;
	vertical[1][0] = 90;
	vertical[1][1] = 90 + angle_diff;
	vertical[2][0] = 270 - angle_diff;
	vertical[2][1] = 270;
	vertical[3][0] = 270;
	vertical[3][1] = 270 + angle_diff;

	// non horizontal and non vertical lines
	vector<Vec4i> leaning_lines;
	vector<Vec4i> horizontal_lines;
	vector<Vec4i> vertical_lines;

	for (size_t i = 0; i < lines.size(); i++){
		Point pt1, pt2;
		bool horizontal_line_found = false;
		bool vertical_line_found = false;
		pt1.x = lines[i][0];
		pt1.y = lines[i][1];
		pt2.x = lines[i][2];
		pt2.y = lines[i][3];
		angle.push_back(fastAtan2(pt2.x-pt1.x, pt2.y - pt1.y));

		//horizontal line check; angle in interval <0-angle_diff, 0+angle_diff> or <180-angle_diff, 180 + angle_diff>
		for(int j = 0; j < 4; j++){
			if (angle[i] >= horizontal[j][0] && angle[i] <= horizontal[j][1]) horizontal_line_found = true;
		}
		//vertical line check; angle in interval <90-angle_diff, 90+angle_diff> or <270-angle_diff, 270 + angle_diff>
		for(int j = 0; j < 4; j++){
			if (angle[i] >= vertical[j][0] && angle[i] <= vertical[j][1]) vertical_line_found = true;
		}

		if (horizontal_line_found){
			Vec4i line(pt1.x, pt1.y, pt2.x, pt2.y);
			horizontal_lines.push_back(line);
		}

		if (vertical_line_found){
			Vec4i line(pt1.x, pt1.y, pt2.x, pt2.y);
			vertical_lines.push_back(line);
		}
		if (!vertical_line_found && !horizontal_line_found){
			Vec4i line(pt1.x, pt1.y, pt2.x, pt2.y);
			leaning_lines.push_back(line);
			//cout << i << " " << angle[i] << endl;
		}
	}
	lines = leaning_lines;




	for(size_t i=0; i<lines.size(); i++)
	{
		Point pt1, pt2;
		pt1.x = lines[i][0];
		pt1.y = lines[i][1];
		pt2.x = lines[i][2];
		pt2.y = lines[i][3];

		line(outputImg, pt1, pt2, CV_RGB(0,0,0), 1);
		/*circle(outputImg, pt1, 2, CV_RGB(255,255,255), CV_FILLED);
		 *                 circle(outputImg, pt1, 3, CV_RGB(0,0,0),1);
		 *                                 circle(outputImg, pt2, 2, CV_RGB(255,255,255), CV_FILLED);
		 *                                                 circle(outputImg, pt2, 3, CV_RGB(0,0,0),1);*/

		// Store into vector of pairs of Points for msac
		aux.clear();
		aux.push_back(pt1);
		aux.push_back(pt2);
		lineSegments.push_back(aux);
	}
	// Multiple vanishing points
	//std::vector<cv::Mat> vps;                       // vector of vps: vps[vpNum], with vpNum=0...numDetectedVps
	std::vector<std::vector<int> > CS;      // index of Consensus Set for all vps: CS[vpNum] is a vector containing indexes of li
	std::vector<int> numInliers;

	//std::vector<std::vector<std::vector<cv::Point> > > lineSegmentsClusters;
	lineSegmentsClusters.clear();

	// Call msac function for multiple vanishing point estimation
	msac.multipleVPEstimation(lineSegments, lineSegmentsClusters, numInliers, vps, numVps);

	bool left_line_exist = false;
	bool right_line_exist = false;
	if (lineSegmentsClusters.size() != 0){
		for (size_t i = 0; i < lineSegmentsClusters[0].size(); i++){
			if (lineSegmentsClusters[0][i][0].x < imgGRAY.cols/2 || lineSegmentsClusters[0][i][1].x < imgGRAY.cols/2){
				left_line_exist = true;
			}
			if (lineSegmentsClusters[0][i][0].x >= imgGRAY.cols/2 || lineSegmentsClusters[0][i][1].x >= imgGRAY.cols/2){
				right_line_exist = true;
			}
		}
	}
	if (!(left_line_exist && right_line_exist)){
		FILE_LOG(logDEBUG) << "VanishingPoint is not correct";
		vps.clear();
	}



	FILE_LOG(logDEBUG) << "Vanshing Point detected vps.size()=" << vps.size();

	for(unsigned int v=0; v<vps.size(); v++)
	{
		//printf("VP %d (%.3f, %.3f, %.3f)", v, vps[v].at<float>(0,0), vps[v].at<float>(1,0), vps[v].at<float>(2,0));
		FILE_LOG(logDEBUG) << "VP: (" << v << ") [" << vps[v].at<float>(0,0) << ", " << vps[v].at<float>(1,0) << ", " << vps[v].at<float>(2,0) << "]";
		//fflush(stdout);
		double vpNorm = cv::norm(vps[v]);
		if(fabs(vpNorm - 1) < 0.001)
		{
			//printf("(INFINITE)");
			//fflush(stdout);
		}
		//printf("\n");
	}



	// Draw line segments according to their cluster
	msac.drawCS(outputImg, lineSegmentsClusters, vps);

}

void VanishingPoint::init(int numVps, int width, int height) {
	mode =  MODE_NIETO;
	this->numVps = numVps;
	verbose = false;
	procSize = Size(width, height);
	msac.init(mode, procSize, verbose);
}

void VanishingPoint::processImage(int numVps, cv::Mat& imgGRAY, cv::Mat& outputImg, vector<Mat>& vps) {
	processImage(this->msac, numVps, imgGRAY, outputImg, vps);
}

/*
void VanishingPoint::getVanishingPoints(vector<cv::Point2f>& vps) {
	for(unsigned int i = 0; i < vps_points.size(); i++){
		Point2f point;
		point.x = vps_points[i].at<float>(0,0);
		point.y = vps_points[i].at<float>(1,0);
		//point.z = vps_points[i].at<float>(2,0);
		vps.push_back(point);
	}
}
 */

VanishingPoint::VanishingPoint() : minCannyThreshold(180), maxCannyThreshold(120){
}

void VanishingPoint::getlineSegmentsClusters(std::vector<std::vector<std::vector<cv::Point> > >& line_segments) {
	line_segments = lineSegmentsClusters;
}

VanishingPoint::~VanishingPoint() {
}

