/*#include <iostream>
#include <sstream>
#include <string>
#include <math.h>
#include "opencv2/core/core.hpp"
#include "opencv2/highgui/highgui.hpp"
#include "opencv2/imgproc/imgproc.hpp"

#include "agents.h"
#include "MSAC.h"
 */

#include "agentspace.h"
#include "AgentBenchmark.h"
#include "AgentCameraRead.h"
#include "AgentChassisCommand.h"
#include "AgentCorrection.h"
#include "AgentDirectionFinder.h"
#include "AgentGUI.h"
#include "AgentHistogram.h"
#include "AgentLaplacian.h"
#include "AgentLineSegments.h"
#include "AgentMatcher.h"
#include "AgentObstacle.h"
#include "AgentPNGRead.h"
#include "AgentPNGWrite.h"
#include "AgentReadWrite.h"
#include "AgentReferenceImageSaver.h"
#include "AgentSteerControl.h"
#include "AgentTrigger.h"
#include "AgentVanishingPoint.h"
#include "AgentVanishingPointChecker.h"
#include "AgentVanishingPointControl.h"
#include "AgentVanishingPointCounter.h"


using namespace std;
using namespace cv;

const double PI = atan(1.0)*4;
int main () {
	FILELog::ReportingLevel() = FILELog::FromString("DEBUG");
	vector<Agent*> agents;
	Config cfg;
	cfg.width = DEFAULT_WIDTH;
	cfg.height = DEFAULT_HEIGHT;
	cfg.command = CHASSIS_COMMAND::FORWARD;

	Config cfg_right;
	cfg_right.movement1 = MOVEMENT::RIGHT;
	cfg_right.movement2 = MOVEMENT::LEFT;

	Config cfg_pngwrite1;
	Config cfg_pngwrite2;
	Config cfg_pngwrite3;
	cfg_pngwrite1.block_write = "INPUT_Mat";
	cfg_pngwrite2.block_write = "Vanishing_Mat";
	cfg_pngwrite3.block_write = "MATCHED_Mat";

    //agents.push_back(new AgentPNGRead(cfg));
    agents.push_back(new AgentCameraRead(cfg));
    agents.push_back(new AgentGUI(cfg));
    //agents.push_back(new AgentObstacle(cfg));
    agents.push_back(new AgentSteerControl(cfg));
    agents.push_back(new AgentVanishingPoint(cfg));
    agents.push_back(new AgentVanishingPointControl(cfg));
    //agents.push_back(new AgentChassisCommand(cfg));
    agents.push_back(new AgentVanishingPointChecker(cfg));
    agents.push_back(new AgentVanishingPointCounter(cfg));
    agents.push_back(new AgentBenchmark(cfg));
    agents.push_back(new AgentReadWrite(cfg));
    agents.push_back(new AgentPNGWrite(cfg_pngwrite1));
    agents.push_back(new AgentPNGWrite(cfg_pngwrite2));
    agents.push_back(new AgentPNGWrite(cfg_pngwrite3));
    agents.push_back(new AgentDirectionFinder(cfg_right));
    //agents.push_back(new AgentHistogram(cfg));
    //agents.push_back(new AgentLineSegments(cfg));
    agents.push_back(new AgentMatcher(cfg));
    agents.push_back(new AgentCorrection(cfg));
    agents.push_back(new AgentReferenceImageSaver(cfg));

	for (unsigned int i = 0; i < agents.size(); i++){
		agents[i]->start();

	}
	for (unsigned int i = 0; i < agents.size(); i++){
		agents[i]->join();
	}
	FILE_LOG(logDEBUG) << "Agentspace finished !";
}
