#ifndef agentspace_thread_INCLUDED
#define agentspace_thread_INCLUDED

#include <iostream>
#include <cstdio>
using namespace std;

#include "pthread.h"
#include "log.h"

#ifdef __MINGW32__
void delay (int ms);
#endif

class Thread {
	private:
		pthread_t _id;
		static unsigned int nid;
		unsigned int _nid;
		Thread(const Thread& arg);
		Thread& operator=(const Thread& rhs);
	protected:
		bool started;
		void *args;
		static void *exec(void *thr);
	public:
		Thread(void *args=NULL);
		virtual ~Thread();
		unsigned int tid() const;
		int start(void *args=NULL);
		void join();
		virtual void run(void *args=NULL) = 0;
		void delay (int ms);
};

class Synchronize {
	private:
		pthread_mutex_t mutex;
		void *last_msg;
		Synchronize(const Synchronize& arg);
		Synchronize& operator=(const Synchronize& rhs);
	public:
		Synchronize();
		~Synchronize();
		void lock();
		void unlock();
};

class Monitor {
	private:
		pthread_mutex_t mutex;
		pthread_cond_t cond;
		void *last_msg;
		Monitor(const Monitor& arg);
		Monitor& operator=(const Monitor& rhs);
	public:
		Monitor();
		~Monitor();
		void *wait();
		void *wait(struct timespec *tim);
		void notify(void *msg=NULL);
};

class Proxy {
	private:
		static unsigned int nid;
		unsigned int _nid;
		Monitor *_monitor;
		int status;
	public:
		Proxy(Monitor& monitor);
		~Proxy();
		unsigned int pid() const;
		void trigger();
		static Proxy *receive(Monitor& monitor);
		bool operator==(const Proxy& rhs) const;
		bool is(int status_mask);
		void set(int status_mask);
};

#endif
