#ifndef agentspace_timer_INCLUDED
#define agentspace_timer_INCLUDED

#include "thread.h"
#include <sys/time.h>
#include <queue>





class Timer {
	private:
		static unsigned int nid;
		unsigned int _nid;
		Proxy *proxy;
        struct timespec tim;
        int period;
        bool active;
	public:
		Timer(Proxy& proxy);
		bool operator<(const Timer& rhs) const;
		bool operator==(const Timer& rhs) const;
		unsigned int tid();
		struct timespec *getTiming();
		bool isActive();
		bool isCyclic();
		int getPeriod();
		void start(int ms_offset, int ms_period=0, int round_ms=-1);
		void start(struct timespec *abstime, int ms_period=0);
		void stop();
		void trigger();
		static void get_time(struct timespec *now);
		static void inc_time(struct timespec *now, int ms);
		static bool cmp_time(struct timespec *tim, struct timespec *limit);
        friend ostream& operator<<(ostream &os,Timer &timer);
};

//------------------------ Timer * comparison --------------------

struct CompareTimer : public std::binary_function<Timer*, Timer*, bool> {
  bool operator()(const Timer* lhs, const Timer* rhs) const {
    return *lhs < *rhs;
  }
};

//------------------------ TimerThread ---------------------------

class TimerThread : public Thread {
    private:
        static TimerThread* singleton;
        static Synchronize* p_sync;
        std::priority_queue<Timer*, vector<Timer*>, CompareTimer> queue;
        Monitor monitor;
        TimerThread() : Thread(NULL) {};
        TimerThread(const TimerThread&){};
        void operator=(const TimerThread&){};
        ~TimerThread() {};
    public:
        static TimerThread *getInstance();
        void run (void *args=NULL);
        void enque(Timer *task);
        void erase(Timer *task);
};


#endif
